use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007003
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Alexandr
Breno
Caleb
Ciornii
Cushing
David
Dist
Etheridge
Golden
Graham
Gruenauer
Harley
Karen
Knop
Marcel
Oliveira
Pig
Plugin
PodSpelling
PodSpellingTests
Randy
Stauner
Test
Zilla
alexchorny
dagolden
de
ether
garu
haarg
hanekomu
harleypig
irc
lib
rwstauner
xenoterracide
